"Change Set:		7837ST80-dtl.54

ST80-dtl.54:
Implement MVCProject>>addDeferredUIMessage: and move implementation for deferred UI messages from DeferredActionStandardSystemController to Controller. This allows scheduling deferred of deferred UI actions regardless of which controller is currently active. This is important when, for example, deferred actions are scheduled during system startup and a ScreenController is the active controller (ProcessBrowser may do this during startup). Reference Mantis 1578.


ST80-ar.53:
Project refactoring: Provide MVCProject to represent 'classic' Smalltalk-80 environments.

"!
Object subclass: #Controller
	instanceVariableNames: 'model view sensor deferredActionQueue lastActivityTime'
	classVariableNames: 'MinActivityLapse'
	poolDictionaries: ''
	category: 'ST80-Kernel-Remnants'!

Project subclass: #MVCProject
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ST80-Support'!

!ScreenController methodsFor: 'menu messages' stamp: 'ar 9/18/2009 20:08'!
openMorphicProject
	"Open a morphic project from within a MVC project"
	Smalltalk at: #MorphicProject ifPresent:[:mp| ProjectView open: mp new].! !

!MVCProject methodsFor: 'display' stamp: 'ar 9/18/2009 20:27'!
displaySizeChanged
	"Inform the current project that its display size has changed"
	world "ScheduledControllers" restore; searchForActiveController! !

!Controller methodsFor: 'control defaults' stamp: 'dtl 4/22/2004 22:58'!
controlActivity
	"Pass control to the next control level (that is, to the Controller of a 
	subView of the receiver's view) if possible. It is sent by 
	Controller|controlLoop each time through the main control loop. It should 
	be redefined in a subclass if some other action is needed."

	[self deferredActionQueue isEmpty]
		whileFalse: [deferredActionQueue next value].
	self controlToNextLevel! !

!MVCProject methodsFor: 'initialize' stamp: 'ar 9/18/2009 21:32'!
openProject: aProject
	"Create a new for a new project in the context of the receiver"
	ProjectView openAndEnter: aProject.! !

!MVCProject methodsFor: 'utilities' stamp: 'ar 9/18/2009 20:19'!
subProjects
	"Answer a list of all the subprojects  of the receiver. "
	^(world controllersSatisfying: [:m | m model isKindOf: Project]) 
		collect: [:c | c model]! !

!MVCProject class methodsFor: 'class initialization' stamp: 'ar 9/18/2009 20:43'!
convertOldProjects
	"Convert old MVC projects to be MVCProjects"
	Project allInstancesDo:[:p|
		(p world isKindOf: ControlManager) 
			ifTrue:[p primitiveChangeClassTo: self basicNew]].! !

!Controller methodsFor: 'basic control sequence' stamp: 'dtl 9/20/2009 18:37'!
deferredActionQueue

	^deferredActionQueue ifNil: [deferredActionQueue := SharedQueue new]! !

!MVCProject methodsFor: 'initialize' stamp: 'ar 9/18/2009 20:31'!
initialize
	super initialize.
	world := ControlManager new.! !

!ScreenController methodsFor: 'menu messages' stamp: 'ar 9/18/2009 21:36'!
openProject 
	"Create and schedule a Project."
	ProjectView open: MVCProject new.! !

!MVCProject methodsFor: 'display' stamp: 'ar 9/18/2009 20:26'!
restore
	"Redraw the entire Project"
	world "ScheduledControllers" restore.! !

!MVCProject methodsFor: 'display' stamp: 'ar 9/18/2009 20:25'!
invalidate
	"Invalidate the entire project so that a redraw will be forced later."
	world "ScheduledControllers" unCacheWindows! !

!MVCProject methodsFor: 'scheduling' stamp: 'dtl 9/22/2009 23:11'!
addDeferredUIMessage: valuableObject 
	"Arrange for valuableObject to be evaluated at a time when the user interface
	is in a coherent state."

	ScheduledControllers activeController
		ifNotNilDo: [:controller | controller addDeferredUIMessage: valuableObject]! !

!Controller methodsFor: 'basic control sequence' stamp: 'dtl 4/19/2004 21:11'!
addDeferredUIMessage: valuableObject 
	"Arrange for valuableObject to be evaluated the next time the 
	controlActivity for this controller becomes active."

	self deferredActionQueue nextPut: valuableObject! !

